----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------
instantWash = {};
instantWash.modDirectory  = g_currentModDirectory;
source(Utils.getFilename("fs19PSUtil.lua", instantWash.modDirectory));
local modDescFile = loadXMLFile("modDesc", instantWash.modDirectory .. "modDesc.xml");
instantWash.title = getXMLString(modDescFile, "modDesc.title."..g_languageShort);
instantWash.author = getXMLString(modDescFile, "modDesc.author");
instantWash.version = getXMLString(modDescFile, "modDesc.version");
delete(modDescFile);
instantWash.inputKey = Input.KEY_insert;
instantWash.washSound = createSample("washSound");
loadSample(instantWash.washSound, instantWash.modDirectory.."washSound.ogg", false);

function instantWash.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations);
end;

function instantWash.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", instantWash);
end;

function instantWash:onLoad(savegame)
	self.spec_instantWash = {};
	local spec = self.spec_instantWash;
	spec.instantWashEvents = {};
	spec.instantWashEventIds = {};	
	spec.washTime = 0;
	spec.washEffect = {};
	spec.washEffect.isActive = false;
	local scaleFactor = 1.0;
	local y = 1.8;
	if self.spec_wheels ~= nil and #self.spec_wheels.wheels > 0 then
		y = y*2; 
		if self.size.height ~= nil then y = self.size.height; y = y-0.25; end;
	end;
	local psData = {};
	psData.psFile = "particleSystems/wfxSplashes.i3d";
	psData.posX = 0;
	psData.posY = y;
	psData.posZ = 1.0;
	psData.worldSpace = false;
	local loaded = FS19PSUtil.loadParticleSystemFromData(psData, spec.washEffect, nil, false, nil, instantWash.modDirectory, self.components[1].node);
	setScale(spec.washEffect.shape, scaleFactor, scaleFactor, scaleFactor);
end;

function instantWash:onUpdate(dt)
	local vehicle = self:getRootVehicle();
	local spec = self.spec_instantWash;
	local rootSpec = vehicle.spec_instantWash;
	if rootSpec.washEffect ~= nil and rootSpec.washEffect.isActive then
		if rootSpec.washTime > 0 then
			for _, nodeData in ipairs(self.spec_washable.washableNodes) do
				self:setNodeDirtAmount(nodeData, 0, true);
				ParticleUtil.setEmittingState(spec.washEffect, true);
			end;
			rootSpec.washTime = rootSpec.washTime-1;
		else
			rootSpec.washEffect.isActive = false;
		end;
	else
		ParticleUtil.setEmittingState(spec.washEffect, false);
		rootSpec.washTime = 0;
	end;
end;


function instantWash:onLeaveVehicle()
	local spec = self.spec_instantWash;
	if spec.washEffect ~= nil then
		if spec.washEffect.isActive or spec.washTime > 0 then
			ParticleUtil.setEmittingState(spec.washEffect, false);
			spec.washEffect.isActive = false;
			spec.washTime = 0;
		end;
	end;
end;

function instantWash:toggleActive(actionName, keyStatus, arg3, arg4, arg5)
	if actionName == "instantWash_washKey" then
		local spec = self:getRootVehicle().spec_instantWash;
		if not spec.washEffect.isActive then
			if not isSamplePlaying(instantWash.washSound) then playSample(instantWash.washSound,1,1,0,0,0); end;
		end;
		spec.washEffect.isActive = true;
		spec.washTime = math.min(spec.washTime+2, 10);
	end;
end;

function instantWash:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		local spec = self.spec_instantWash;
		self:clearActionEventsTable(spec.instantWashEvents);
		spec.instantWashEventIds = {};
        if self:getIsActiveForInput(true, true) then
			local _, actionEventId = self:addActionEvent(spec.instantWashEvents, 'instantWash_washKey', self, instantWash.toggleActive, false, true, true, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, false);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW);
			table.insert(spec.instantWashEventIds, actionEventId);
		end;
	end;
end;

print(instantWash.title.." : v"..instantWash.version.." by "..instantWash.author.." activated");